import json
import pandas as pd
import matplotlib.pyplot as plt
from gboml import GbomlGraph
import matplotlib.dates as mdates
from matplotlib.dates import DateFormatter
import numpy as np

date_rng = pd.date_range(start='2020-01-01', end='2020-12-31', freq='H')

with open("pv_bat_house_AvoidedCostCapex.json") as json_file:
    bat_house_solution = json.load(json_file)
    json_file.close() 

plt.rc('font', size=16)
plt.rc('font', size=16)          
plt.rc('axes', titlesize=16)     
plt.rc('xtick', labelsize=16)    
plt.rc('ytick', labelsize=16)   
plt.rc('legend', fontsize=16)   
plt.rc('figure', titlesize=20)

bill = []
for idx in range(len(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["price_paid"]["values"])):
    bill.append(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["price_paid"]["values"][idx]-bat_house_solution["solution"]["elements"]["Solar_PV"]["variables"]["avoided_cost"]["values"][idx]*(bat_house_solution["solution"]["elements"]["Solar_PV"]["variables"]["pvelectricity_generated"]["values"][idx] + bat_house_solution["solution"]["elements"]["Battery"]["variables"]["battery_output"]["values"][idx]-bat_house_solution["solution"]["elements"]["House"]["variables"]["elec_demand"]["values"][idx]))


fig, ax1 = plt.subplots(figsize=(16, 9), dpi=80)
ax1.plot(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["electricity_exchanged"]["values"][212*24:24*213], label='Utility Electricity', color='blue')
ax1.plot(bat_house_solution["solution"]["elements"]["House"]["variables"]["elec_demand"]["values"][212*24:24*213], label ='Electricity Demand', color='black')
ax1.plot(bat_house_solution["solution"]["elements"]["Battery"]["variables"]["state_of_charge"]["values"][212*24:24*213], label ='State of Charge [kWh]', color='orange')
ax1.plot(bat_house_solution["solution"]["elements"]["Solar_PV"]["variables"]["pvelectricity_generated"]["values"][212*24:24*213], label='PV Generated Electricity', color='green')
# Battery Charging & Discharging -- makes the plot a bit too busy
#ax1.plot(bat_house_solution["solution"]["elements"]["Battery"]["variables"]["battery_output"]["values"][12*24:24*13], label ='Battery Discharging [kWh]', color='orange')
#ax1.plot([-value for value in bat_house_solution["solution"]["elements"]["Battery"]["variables"]["battery_input"]["values"][12*24:24*13]], label ='Battery Charging [kWh]', color='orange')
ax1.set_ylim(-(max(np.abs(bat_house_solution["solution"]["elements"]["House"]["variables"]["elec_demand"]["values"][212*24:24*213]))),max(np.abs(bat_house_solution["solution"]["elements"]["House"]["variables"]["elec_demand"]["values"][212*24:24*213])))
ax1.set_xlabel("Time [hours]")
ax1.set_ylabel('Energy [kWh]')
ax1.legend(loc='upper center',bbox_to_anchor=(0.5,-.075),ncol=4)
plt.grid(True, color='grey', linestyle=':', zorder=0)
ax2 = ax1.twinx()
ax2.step(range(len(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["price_paid"]["values"][212*24:24*213])),bat_house_solution["model"]["nodes"]["Grid"]["parameters"]["elec_price"][12*24:24*13], where='pre', label='TOU Price', color = 'grey', alpha=0.1)
ax2.plot(bill[212*24:24*213], label='Price Paid', color = 'red')
ax2.set_ylabel("$/kWh")
#ax2.set_ylim(-max(np.abs(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["price_paid"]["values"][12*24:24*13])),max(np.abs(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["price_paid"]["values"][12*24:24*13])))
ax2.set_ylim(-max(np.abs(bill[212*24:24*213])),max(np.abs(bill[212*24:24*213])))
plt.fill_between(range(len(bill[212*24:24*213])),bat_house_solution["model"]["nodes"]["Grid"]["parameters"]["elec_price"][212*24:24*213], 0, step='pre',interpolate=True, color='grey', alpha=0.1)
ax2.legend(loc='upper center',bbox_to_anchor=(.8,-.02), ncol=2)
plt.title('24-Hour Electricity Dispatch')

fig, ax1 = plt.subplots(figsize=(16, 9), dpi=80)
ax1.plot(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["electricity_exchanged"]["values"][212*24:24*219], label='Utility Electricity', color='blue')
ax1.plot(bat_house_solution["solution"]["elements"]["House"]["variables"]["elec_demand"]["values"][212*24:24*219], label ='Electricity Demand', color='black')
ax1.plot(bat_house_solution["solution"]["elements"]["Battery"]["variables"]["state_of_charge"]["values"][212*24:24*219], label ='State of Charge [kWh]', color='orange')
ax1.plot(bat_house_solution["solution"]["elements"]["Solar_PV"]["variables"]["pvelectricity_generated"]["values"][212*24:24*219], label='PV Generated Electricity', color='green')
# Battery Charging & Discharging -- makes the plot a bit too busy
#ax1.plot(bat_house_solution["solution"]["elements"]["Battery"]["variables"]["battery_output"]["values"][0:24*7], label ='Battery Discharging [kWh]', color='orange')
#ax1.plot([-value for value in bat_house_solution["solution"]["elements"]["Battery"]["variables"]["battery_input"]["values"][0:24*7]], label ='Battery Charging [kWh]', color='orange')
ax1.set_ylim(-(max(np.abs(bat_house_solution["solution"]["elements"]["House"]["variables"]["elec_demand"]["values"][212*24:24*219]))),max(np.abs(bat_house_solution["solution"]["elements"]["House"]["variables"]["elec_demand"]["values"][212*24:24*219])))
ax1.set_xlabel("Time [hours]")
ax1.set_ylabel('Energy [kWh]')
ax1.legend(loc='upper center',bbox_to_anchor=(0.5,-.075),ncol=4)
plt.grid(True, color='grey', linestyle=':', zorder=0)
ax2 = ax1.twinx()
ax2.step(range(len(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["price_paid"]["values"][212*24:24*219])),bat_house_solution["model"]["nodes"]["Grid"]["parameters"]["elec_price"][212*24:24*219], where='pre', label='TOU Price', color = 'grey', alpha=0.1)
ax2.plot(bill[212*24:24*219], label='Price Paid', color = 'red')
ax2.set_ylabel("$/kWh")
ax2.set_ylim(-max(np.abs(bill[212*24:24*219])),max(np.abs(bill[212*24:24*219])))
#ax2.set_ylim(-max(np.abs(bat_house_solution["model"]["nodes"]["Grid"]["parameters"]["elec_price"][0:24*7])),max(np.abs(bat_house_solution["model"]["nodes"]["Grid"]["parameters"]["elec_price"][0:24*7])))
plt.fill_between(range(len(bill[212*24:24*219])),bat_house_solution["model"]["nodes"]["Grid"]["parameters"]["elec_price"][212*24:24*219], 0, step='pre',interpolate=True, color='grey', alpha=0.1)
ax2.legend(loc='upper center',bbox_to_anchor=(.8,-.02), ncol=2)
plt.title('One Week of Electricity Dispatch')

print('For a house with a yearly demand [kWh] of')
print(sum(bat_house_solution["solution"]["elements"]["House"]["variables"]["elec_demand"]["values"]))
print('With battery capex [$/kWh] of')
print(bat_house_solution["model"]["nodes"]["Battery"]["parameters"]["capex_batt"])
print('Battery capacity [kWh] is ')
print(bat_house_solution["solution"]["elements"]["Battery"]["variables"]["battery_capacity"]["values"])
#print(bat_house_solution["model"]["nodes"]["Battery"]["parameters"]["battery_capacity"])
print('With solar capex [$/kWh] of')
print(bat_house_solution["model"]["nodes"]["Solar_PV"]["parameters"]["capex_pv"])
print('Solar capacity [kW] is ')
print(bat_house_solution["solution"]["elements"]["Solar_PV"]["variables"]["pv_capacity"]["values"])
#print(bat_house_solution["model"]["nodes"]["Solar_PV"]["parameters"]["pv_capacity"])
print('Total electricity exchanged [kWh] was')
print(sum(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["electricity_exchanged"]["values"]))
print('and the total energy supplied to the house [kWh] by the battery was')
print(sum(bat_house_solution["solution"]["elements"]["Battery"]["variables"]["battery_output"]["values"]))
print('and the total energy supplied to the battery was [kWh]')
print(sum(bat_house_solution["solution"]["elements"]["Battery"]["variables"]["battery_input"]["values"]))
print('and the total energy generated by the PV array over the year [kWh]')
print(sum(bat_house_solution["solution"]["elements"]["Solar_PV"]["variables"]["pvelectricity_generated"]["values"]))
print('Total utility bill [$] for the year was')
print(sum(bill))
print('Total (price paid) utility bill [$] for the year was')
print(sum(bat_house_solution["solution"]["elements"]["Grid"]["variables"]["price_paid"]["values"]))

# To run the gboml model:
# gboml C:\Users\kerb930\localRepos\gboml\gboml\pv_bat_house_AvoidedCostCapex.txt --clp --json --output pv_bat_house_AvoidedCostCapex